<?php

use EventPortal\Domain\Component\Option;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Team\Models\Team;
use Illuminate\Support\Str;

/** @var \Illuminate\Database\Eloquent\Factory $factory */

$factory->define(Event::class, static function ($faker) {
    $eventName = $faker->word;
    return [
        'name' => $eventName,
        'status' => 'published',
        'slug' => Str::slug($eventName),
        'is_virtual' => Option::make('No')->value(0),
        'type' => json_encode([
            'label' => 'Default',
            'value' => 'default',
        ]),
    ];
});

$factory->state(Event::class, 'snapshot', static function () {
    return [
        'name' => 'Event Name',
        'slug' => 'event-name',
    ];
});

$factory->state(Event::class, 'draft', static function ($faker) {
    return [
        'status' => 'draft',
    ];
});

$factory->state(Event::class, 'published', static function ($faker) {
    return [
        'status' => 'published',
    ];
});

$factory->state(Event::class, 'post', static function ($faker) {
    return [
        'status' => 'post',
    ];
});

$factory->state(Event::class, 'cancelled', static function ($faker) {
    return [
        'status' => 'cancelled',
    ];
});

$factory->state(Event::class, 'archived', static function ($faker) {
    return [
        'status' => 'archived',
    ];
});

$factory->state(Event::class, 'has-team', []);

$factory->afterCreatingState(Event::class, 'has-team', static function (Event $event): void {
    $team = factory(Team::class)->state('snapshot')->create();

    $event->teams()->sync($team);
});

$factory->state(Event::class, 'meta', static function ($faker): array {
    return [
        'snippet' => $faker->words(10, true),
        'description' => $faker->sentences(4, true),
    ];
});

$factory->state(Event::class, 'virtual', static function ($faker): array {
    return [
        'is_virtual' => Option::make('Yes')->value(1),
        'broadcast_URL' => $faker->url,
    ];
});
