<?php

use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Component\Dropdown\Select;
use EventPortal\Domain\Component\Input\Text;
use EventPortal\Domain\Component\Section;
use EventPortal\Domain\Event\Form\Handlers\CreateForm;
use EventPortal\Domain\Event\Form\Models\Form as FormModel;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Enums\InertiaMethod;

Form::make('form.create')
    ->addHandler(CreateForm::class)
    ->addFields(static function () {
        return [
            Section::make()->fields([
                Text::make('Name of form')
                    ->name('name')
                    ->rules(['required'])
                    ->placeholder('Register to attend'),
                Select::make('Select a type of form')
                    ->name('type')
                    ->rules(['required'])
                    ->placeholder('Please select a form type')
                    ->helpText('Choose \'Registration\' to create your delegate registration form, or \'Decline\' to 
                        create a  form for your delegates to provide information on why they are declining an invite.')
                    ->options(FormModel::typesAsOptions()),
            ]),
        ];
    })
    ->addActions(static function (Event $event) {
        return [
            Button::make('Cancel')
                ->variant('secondary')
                ->method(InertiaMethod::VISIT())
                ->route('admin.form.index', ['event' => $event]),
            Button::make('Save')
                ->method(InertiaMethod::POST())
                ->route('admin.form.store', ['event' => $event]),
        ];
    });
