<?php

namespace EventPortal\Domain\Agenda;

use EventPortal\Domain\Agenda\Models\Agenda;
use EventPortal\Domain\Agenda\Models\Pathway;
use EventPortal\Domain\Agenda\Values\Session;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Event\Models\EventDelegate;
use Spatie\Period\Boundaries;
use Spatie\Period\Period;
use Spatie\Period\Precision;

class Sessions
{
    private array $sessions;

    private Agenda $agenda;

    private Delegate $delegate;

    public function __construct(Agenda $agenda, Delegate $delegate)
    {
        $this->agenda = $agenda;
        $this->delegate = $delegate;

        $this->calculate();
    }

    public function calculate(): void
    {
        $agenda = $this->agenda;
        $delegate = $this->delegate;

        $eventDelegate = EventDelegate::forEventAndDelegate($agenda->event, $delegate)->first();

        $defaultPathway = Pathway::query()
            ->with(['sessions' => function ($query) use ($agenda) {
                return $query->where('agenda_id', $agenda->id);
            }, 'sessions.eventSpeakers', 'sessions.eventVenue'])
            ->defaultForEvent($agenda->event)
            ->first();

        $nonDefaultPathway = Pathway::query()
            ->with(['sessions' => function ($query) use ($agenda) {
                return $query->where('agenda_id', $agenda->id);
            }, 'sessions.eventSpeakers', 'sessions.eventVenue'])
            ->nonDefaultForEventDelegate($eventDelegate)
            ->first();

        if ($nonDefaultPathway === null) {
            $this->sessions = $defaultPathway
                ->sessions
                ->sortBy('starting_at')
                ->mapInto(Session::class)
                ->values()
                ->toArray();

            return;
        }

        $nonDefaultPathwayPeriods = $nonDefaultPathway
            ->sessions
            ->map(function (Models\Session $session) {
                return new Period(
                    $session->starting_at->toDateTimeImmutable(),
                    $session->ending_at->toDateTimeImmutable(),
                    Precision::SECOND,
                    Boundaries::EXCLUDE_ALL
                );
            })
            ->toArray();

        $sessions = $defaultPathway
            ->sessions
            ->reject(function (Models\Session $session) use ($nonDefaultPathwayPeriods) {
                $period = new Period(
                    $session->starting_at->toDateTimeImmutable(),
                    $session->ending_at->toDateTimeImmutable(),
                    Precision::SECOND,
                    Boundaries::EXCLUDE_ALL
                );

                return $period->overlap(...$nonDefaultPathwayPeriods)->isEmpty() === false;
            });

        $this->sessions = $nonDefaultPathway
            ->sessions
            ->merge($sessions)
            ->sortBy('starting_at')
            ->mapInto(Session::class)
            ->values()
            ->toArray();
    }

    public function get(): array
    {
        return $this->sessions;
    }
}
