<?php

namespace EventPortal\Domain\User\Notifications;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Notifications\Messages\MailMessage;
use Illuminate\Notifications\Notification;
use Illuminate\Support\Facades\Lang;

class ResetPasswordNotification extends Notification implements ShouldQueue
{
    use Queueable;

    public string $token;

    /**
     * @var \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\Routing\UrlGenerator|string
     */
    public $url;

    public string $key = 'auth.password-reset';

    public function __construct(string $token)
    {
        $this->token = $token;
    }

    public function via(): array
    {
        return ['mail'];
    }

    /**
     * @param mixed $notifiable
     */
    public function toMail($notifiable): MailMessage
    {
        $url = url(route('password.reset', [
            'token' => $this->token,
            'email' => $notifiable->getEmailForPasswordReset(),
        ], false));

        $data = [
            'url' => $url,
            'recipient' => $notifiable,
        ];

        return (new MailMessage)
            ->subject(Lang::get('Reset Password Notification'))
            ->view([
                'EventPortal::email.' . $this->key,
                "EventPortal::email.{$this->key}-plain",
            ], $data);
    }

    public function toArray(): array
    {
        return [
            'token' => $this->token,
        ];
    }
}
