<?php

namespace EventPortal\Http\Middleware;

use Closure;
use EventPortal\Contracts\Event;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\Models\EventDelegate;
use Illuminate\Http\Request;
use Illuminate\Http\Response;

class CanDelegateViewForm
{
    /**
     * Handle an incoming request.
     *
     * @return mixed
     */
    public function handle(Request $request, Closure $next)
    {
        abort_unless(
            $this->delegateCanView($request->event, $request->form, auth()->user()->delegate),
            Response::HTTP_FORBIDDEN
        );

        return $next($request);
    }

    private function delegateCanView(Event $event, Form $form, Delegate $delegate): bool
    {
        $eventDelegate = EventDelegate::forEventAndDelegate($event, $delegate)->first();
        $formType = strtolower($form->type);
        $statusesNeeded = config("forms.{$formType}.statusDelegateNeeds");

        return in_array($eventDelegate->status, $statusesNeeded);
    }
}
