<?php

namespace EventPortal\Tests\Unit\ActionList\Handlers;

use EventPortal\Domain\ActionList\ActionList;
use EventPortal\Domain\ActionList\Facades\ActionListTemplate;
use EventPortal\Domain\ActionList\Handlers\AssignActionLists;
use EventPortal\Domain\ActionList\Task;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Form\Payload;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Pipeline\Pipeline;

class AssignActionListsTest extends TestCase
{
    use RefreshDatabase;

    /** @test */
    public function assign_all_action_lists_to_event(): void
    {
        ActionListTemplate::make('default')
            ->addList(
                ActionList::state('draft')
                    ->addTask(function () {
                        return [
                            Task::make('delegates')
                                ->label('At least one delegate needs to be added to the event')
                                ->required()
                                ->href('https://www.google.com', 'Go To Link'),
                            Task::make('venue')
                                ->label('Need to add some details, see link below for them')
                                ->route('something.index', ['id' => 1], 'Go To Route'),
                        ];
                    })
            );

        $event = factory(Event::class)->create();

        $payload = Payload::make()->withModel($event);

        app(Pipeline::class)
            ->send($payload)
            ->through([new AssignActionLists])
            ->thenReturn();

        $this->assertDatabaseHas('action_lists', [
            'status' => 'draft',
            'type' => 'default',
        ]);

        $this->assertDatabaseHas('tasks', [
            'action_list_id' => 1,
            'name' => 'delegates',
            'label' => 'At least one delegate needs to be added to the event',
            'action' => '{"type":"href","route":"https:\/\/www.google.com","label":"Go To Link"}',
        ]);

        $this->assertDatabaseHas('tasks', [
            'action_list_id' => 1,
            'name' => 'venue',
            'label' => 'Need to add some details, see link below for them',
            'action' => '{"type":"route","route":"something.index","routeParams":{"id":1},"label":"Go To Route"}',
        ]);
    }
}
